@@ -24,6 +24,10 @@ package Spreadsheet::Read;
 
  # OO API
  my $book = Spreadsheet::Read->new ("file.csv");
+ my $sheet = $book->sheet (1);
+ my $cell  = $sheet->cell ("A3");
+ my $cell  = $sheet->cell (1, 3);
+
  $book->add ("test.xls");
 
 =cut
@@ -31,7 +35,7 @@ package Spreadsheet::Read;
 use strict;
 use warnings;
 
-our $VERSION = "0.68";
+our $VERSION = "0.70";
 sub  Version { $VERSION }
 
 use Carp;
@@ -109,6 +113,7 @@ my %def_opts = (
     parser  => undef,
     sep     => undef,
     quote   => undef,
+    label   => undef,
     );
 my @def_attr = (
     type    => "text",
@@ -182,12 +187,11 @@ sub sheets {
     wantarray ? sort { $s{$a} <=> $s{$b} } keys %s : $ctrl->{sheets};
     } # sheets
 
-# cr2cell (4, 18) => "D18"
-# No prototype to allow 'cr2cell (@rowcol)'
-sub cr2cell {
+# col2label (4) => "D"
+sub col2label {
     ref $_[0] eq __PACKAGE__ and shift;
-    my ($c, $r) = @_;
-    defined $c && defined $r && $c > 0 && $r > 0 or return "";
+    my $c = shift;
+    defined $c && $c > 0 or return "";
     my $cell = "";
     while ($c) {
 	use integer;
@@ -195,7 +199,16 @@ sub cr2cell {
 	substr $cell, 0, 0, chr (--$c % 26 + ord "A");
 	$c /= 26;
 	}
-    "$cell$r";
+    $cell;
+    } # col2label
+
+# cr2cell (4, 18) => "D18"
+# No prototype to allow 'cr2cell (@rowcol)'
+sub cr2cell {
+    ref $_[0] eq __PACKAGE__ and shift;
+    my ($c, $r) = @_;
+    defined $c && defined $r && $c > 0 && $r > 0 or return "";
+    col2label ($c) . $r;
     } # cr2cell
 
 # cell2cr ("D18") => (4, 18)
@@ -212,7 +225,7 @@ sub cell2cr {
 # my @row = cellrow ($book->[1], 1);
 # my @row = $book->cellrow (1, 1);
 sub cellrow {
-    my $sheet = ref $_[0] eq __PACKAGE__ ? $_[0]->[shift] : shift or return;
+    my $sheet = ref $_[0] eq __PACKAGE__ ? (shift)->[shift] : shift or return;
     ref     $sheet eq "HASH" && exists  $sheet->{cell}   or return;
     exists  $sheet->{maxcol} && exists  $sheet->{maxrow} or return;
     my $row   = shift or return;
@@ -223,7 +236,7 @@ sub cellrow {
 
 # my @row = row ($book->[1], 1);
 sub row {
-    my $sheet = ref $_[0] eq __PACKAGE__ ? $_[0]->[shift] : shift or return;
+    my $sheet = ref $_[0] eq __PACKAGE__ ? (shift)->[shift] : shift or return;
     ref     $sheet eq "HASH" && exists  $sheet->{cell}   or return;
     exists  $sheet->{maxcol} && exists  $sheet->{maxrow} or return;
     my $row   = shift or return;
@@ -234,7 +247,7 @@ sub row {
 # Convert {cell}'s [column][row] to a [row][column] list
 # my @rows = rows ($book->[1]);
 sub rows {
-    my $sheet = ref $_[0] eq __PACKAGE__ ? $_[0]->[shift] : shift or return;
+    my $sheet = ref $_[0] eq __PACKAGE__ ? (shift)->[shift] : shift or return;
     ref    $sheet eq "HASH" && exists $sheet->{cell}   or return;
     exists $sheet->{maxcol} && exists $sheet->{maxrow} or return;
     my $s = $sheet->{cell};
@@ -253,6 +266,11 @@ sub sheet {
 	return bless $book->[$sheet]			=> $class;
     exists $book->[0]{sheet}{$sheet} and
 	return bless $book->[$book->[0]{sheet}{$sheet}]	=> $class;
+    foreach my $idx (1 .. $book->[0]{sheets}) {
+	$book->[$idx]{label} eq $sheet and
+	    return bless $book->[$idx]			=> $class;
+	}
+    return;
     } # sheet
 
 # If option "clip" is set, remove the trailing rows and
@@ -260,8 +278,9 @@ sub sheet {
 sub _clipsheets {
     my ($opt, $ref) = @_;
 
-    if (my $s = $opt->{strip} and $ref->[0]{sheets}) {
-	foreach my $sheet (1 .. $ref->[0]{sheets}) {
+    foreach my $sheet (1 .. $ref->[0]{sheets}) {
+	$ref->[$sheet]{indx} = $sheet;
+	if (my $s = $opt->{strip} and $ref->[0]{sheets}) {
 	    my $ss = $ref->[$sheet];
 	    $ss->{maxrow} && $ss->{maxcol} or next;
 	    foreach my $row (1 .. $ss->{maxrow}) {
@@ -446,7 +465,7 @@ sub ReadData {
 		     : ($io_fil && $txt =~ m/\.(csv)$/i)) {
 	$can{csv} or croak "CSV parser not installed";
 
-	my $label = $io_fil ? $txt : "IO";
+	my $label = defined $opt{label} ? $opt{label} : $io_fil ? $txt : "IO";
 
 	$debug and print STDERR "Opening CSV $label using $can{csv}-", $can{csv}->VERSION, "\n";
 
@@ -799,6 +818,7 @@ sub ReadData {
 	    close   $sc;
 	    }
 	$txt =~ m/\S/ or return;
+	my $label = defined $opt{label} ? $opt{label} : "sheet";
 	my @data = (
 	    {	type	=> "sc",
 		parser	=> "Spreadsheet::Read",
@@ -810,10 +830,10 @@ sub ReadData {
 		    }],
 		error	=> undef,
 		sheets	=> 1,
-		sheet	=> { sheet => 1 },
+		sheet	=> { $label => 1 },
 		},
 	    {	parser	=> 0,
-		label	=> "sheet",
+		label	=> $label,
 		maxrow	=> 0,
 		maxcol	=> 0,
 		cell	=> [],
@@ -935,7 +955,8 @@ sub ReadData {
 sub add {
     my $book = shift;
     my $r = ReadData (@_) or return;
-    $book && (ref $book eq "ARRAY" || ref $book eq __PACKAGE__) && $book->[0]{sheets} or return $r;
+    $book && (ref $book eq "ARRAY" ||
+	      ref $book eq __PACKAGE__) && $book->[0]{sheets} or return $r;
 
     my $c1 = $book->[0];
     my $c2 = $r->[0];
@@ -988,6 +1009,11 @@ sub maxcol {
     return $sheet->{maxcol};
     } # maxrow
 
+sub col2label {
+    my $class = shift;
+    return Spreadsheet::Read::col2label (@_);
+    } # col2label
+
 sub cr2cell {
     my $class = shift;
     return Spreadsheet::Read::cr2cell (@_);
@@ -998,6 +1024,12 @@ sub cell2cr {
     return Spreadsheet::Read::cell2cr (@_);
     } # cell2cr
 
+sub label {
+    my ($sheet, $label) = @_;
+    defined $label and $sheet->{label} = $label;
+    return $sheet->{label};
+    } # label
+
 # my @row = $sheet->cellrow (1);
 sub cellrow {
     my ($sheet, $row) = @_;
@@ -1008,12 +1040,26 @@ sub cellrow {
 
 # my @row = $sheet->row (1);
 sub row {
-    my $sheet = shift;
-    my $row   = shift or return;
-    $row > 0 && $row <= $sheet->{maxrow} or return;
-    map { $sheet->{cr2cell ($_, $row)} } 1..$sheet->{maxcol};
+    my ($sheet, $row) = @_;
+    defined $row && $row > 0 && $row <= $sheet->{maxrow} or return;
+    map { $sheet->{$sheet->cr2cell ($_, $row)} } 1..$sheet->{maxcol};
     } # row
 
+# my @col = $sheet->cellcolumn (1);
+sub cellcolumn {
+    my ($sheet, $col) = @_;
+    defined $col && $col > 0 && $col <= $sheet->{maxcol} or return;
+    my $s = $sheet->{cell};
+    map { $s->[$col][$_] } 1..$sheet->{maxrow};
+    } # cellcolumn
+
+# my @col = $sheet->column (1);
+sub column {
+    my ($sheet, $col) = @_;
+    defined $col && $col > 0 && $col <= $sheet->{maxcol} or return;
+    map { $sheet->{$sheet->cr2cell ($col, $_)} } 1..$sheet->{maxrow};
+    } # column
+
 # Convert {cell}'s [column][row] to a [row][column] list
 # my @rows = $sheet->rows ();
 sub rows {
@@ -1089,7 +1135,8 @@ The data is returned as an array reference:
 To keep as close contact to spreadsheet users, row and column 1 have
 index 1 too in the C<cell> element of the sheet hash, so cell "A1" is
 the same as C<cell> [1, 1] (column first). To switch between the two,
-there are two helper functions available: C<cell2cr ()> and C<cr2cell ()>.
+there are helper functions available: C<cell2cr ()>, C<cr2cell ()>,
+and C<col2label ()>.
 
 The C<cell> hash entry contains unformatted data, while the hash entries
 with the traditional labels contain the formatted values (if applicable).
@@ -1227,12 +1274,23 @@ and higher will dump the entire structure from the back-end parser.
 All other attributes/options will be passed to the underlying parser if
 that parser supports attributes.
 
+=head3 col2label
+
+ my $col_id = col2label (col);
+
+ my $col_id = $book->col2label (col);  # OO
+
+C<col2label ()> converts a C<(column)> (1 based) to the letters used in the
+traditional cell notation:
+
+  my $id = col2label ( 4); # $id now "D"
+  my $id = col2label (28); # $id now "AB"
+
 =head3 cr2cell
 
  my $cell = cr2cell (col, row);
 
  my $cell = $book->cr2cell (col, row);  # OO
- my $cell = $sheet->cr2cell (col, row); # OO
 
 C<cr2cell ()> converts a C<(column, row)> pair (1 based) to the
 traditional cell notation:
@@ -1245,7 +1303,6 @@ traditional cell notation:
  my ($col, $row) = cell2cr ($cell);
 
  my ($col, $row) = $book->cell2cr ($cell);  # OO
- my ($col, $row) = $sheet->cell2cr ($cell); # OO
 
 C<cell2cr ()> converts traditional cell notation to a C<(column, row)>
 pair (1 based):
@@ -1260,7 +1317,6 @@ pair (1 based):
  my @row = Spreadsheet::Read::row ($book->[1], 3);
 
  my @row = $book->row ($sheet, $row); # OO
- my @row = $sheet->row ($row);        # OO
 
 Get full row of formatted values (like C<< $sheet->{A3} .. $sheet->{G3} >>)
 
@@ -1276,7 +1332,6 @@ use argument list, or call it fully qualified.
  my @row = Spreadsheet::Read::cellrow ($book->[1], 3);
 
  my @row = $book->cellrow ($sheet, $row); # OO
- my @row = $sheet->cellrow ($row);        # OO
 
 Get full row of unformatted values (like C<< $sheet->{cell}[1][3] .. $sheet->{cell}[7][3] >>)
 
@@ -1292,7 +1347,6 @@ use argument list, or call it fully qualified or as method call.
  my @rows = Spreadsheet::Read::rows ($book->[1]);
 
  my @rows = $book->rows (1); # OO
- my @rows = $sheet->rows (); # OO
 
 Convert C<{cell}>'s C<[column][row]> to a C<[row][column]> list.
 
@@ -1352,6 +1406,10 @@ Return the numbered or named sheet out of the book. Will return C<undef> if
 there is no match. Will not work for sheets I<named> with a number between 1
 and the number of sheets in the book.
 
+With named sheets will first try to use the list of sheet-labels as stored in
+the control structure. If no match is found, it will scan the actual labels
+of the sheets. In that case, it will return the fist matching sheet.
+
 If defined, the returned sheet will be of class C<Spreadsheet::Read::Sheet>.
 
 =head3 add
@@ -1368,13 +1426,13 @@ If defined, the returned sheet will be of class C<Spreadsheet::Read::Sheet>.
 
  my $col = $sheet->maxcol;
 
-Return the index of the last in-use column in the sheet.
+Return the index of the last in-use column in the sheet. This index is 1-based.
 
 =head3 maxrow
 
  my $row = $sheet->maxrow;
 
-Return the index of the last in-use row in the sheet.
+Return the index of the last in-use row in the sheet. This index is 1-based.
 
 =head3 cell
 
@@ -1384,6 +1442,85 @@ Return the index of the last in-use row in the sheet.
 Return the value for a cell. Using tags will return the formatted value,
 using column and row will return unformatted value.
 
+=head3 col2label
+
+ my $col_id = $sheet->col2label (col);
+
+C<col2label ()> converts a C<(column)> (1 based) to the letters used in the
+traditional cell notation:
+
+  my $id = $sheet->col2label ( 4); # $id now "D"
+  my $id = $sheet->col2label (28); # $id now "AB"
+
+=head3 cr2cell
+
+ my $cell = $sheet->cr2cell (col, row);
+
+C<cr2cell ()> converts a C<(column, row)> pair (1 based) to the
+traditional cell notation:
+
+  my $cell = $sheet->cr2cell ( 4, 14); # $cell now "D14"
+  my $cell = $sheet->cr2cell (28,  4); # $cell now "AB4"
+
+=head3 cell2cr
+
+ my ($col, $row) = $sheet->cell2cr ($cell);
+
+C<cell2cr ()> converts traditional cell notation to a C<(column, row)>
+pair (1 based):
+
+  my ($col, $row) = $sheet->cell2cr ("D14"); # returns ( 4, 14)
+  my ($col, $row) = $sheet->cell2cr ("AB4"); # returns (28,  4)
+
+=head3 col
+
+ my @col = $sheet->column ($col);
+
+Get full column of formatted values (like C<< $sheet->{C1} .. $sheet->{C9} >>)
+
+Note that the indexes in the returned list are 0-based.
+
+=head3 cellcolumn
+
+ my @col = $sheet->cellcolumn ($col);
+
+Get full column of unformatted values (like C<< $sheet->{cell}[3][1] .. $sheet->{cell}[3][9] >>)
+
+Note that the indexes in the returned list are 0-based.
+
+=head3 row
+
+ my @row = $sheet->row ($row);
+
+Get full row of formatted values (like C<< $sheet->{A3} .. $sheet->{G3} >>)
+
+Note that the indexes in the returned list are 0-based.
+
+=head3 cellrow
+
+ my @row = $sheet->cellrow ($row);
+
+Get full row of unformatted values (like C<< $sheet->{cell}[1][3] .. $sheet->{cell}[7][3] >>)
+
+Note that the indexes in the returned list are 0-based.
+
+=head3 rows
+
+ my @rows = $sheet->rows ();
+
+Convert C<{cell}>'s C<[column][row]> to a C<[row][column]> list.
+
+Note that the indexes in the returned list are 0-based, where the
+index in the C<{cell}> entry is 1-based.
+
+=head3 label
+
+ my $label = $sheet->label;
+ $sheet->label ("New sheet label");
+
+Set a new label to a sheet. Note that the index in the control structure will
+I<NOT> be updated.
+
 =head2 Using CSV
 
 In case of CSV parsing, L<C<ReadData>|/ReadData> will use the first line of
@@ -1405,6 +1542,11 @@ See L<Text::CSV_XS|https://metacpan.org/pod/Text-CSV_XS> for documentation.
  my $ss = ReadData ("bad.csv");
  $ss->[0]{error} and say $ss->[0]{error}[1];
 
+As CSV has no sheet labels, the default label for a CSV sheet is its filename.
+For CSV, this can be overruled using the I<label> attribute:
+
+ my $ss = Spreadsheet::Read->new ("/some/place/test.csv", label => "Test");
+
 =head2 Cell Attributes
 X<merged>
 
@@ -1467,6 +1609,8 @@ If a cell itself is not hidden, but the parser holds the information that
 either the row or the column (or both) the field is in is hidden, the flag
 is inherited into the cell attributes.
 
+The entries C<maxrow> and C<maxcol> are 1-based.
+
 =head3 Merged cells
 
 Note that only
@@ -1780,7 +1924,7 @@ H.Merijn Brand, <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005-2016 H.Merijn Brand
+Copyright (C) 2005-2017 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
